// -*- tcl -*-

const irplist = fetch("irplist.txt").then(r => r.text()).then(data => {
    let lines = data.split("\r\n")
    let retval = {}
    for (let i = 0; i < lines.length; i++) {
	[name, def] = lines[i].split("\t")
	if (name != "") {
	    retval[name] = def
	}
    }
    return retval
})

window.addEventListener("load", async () => {
    let h = document.getElementById("busy")
    if (h) h.classList.add("wait")
    try {
	showcapture()
	let w = document.getElementById("families")
	if (!w) return
	signals = await irplist
	for (let name in signals) {
	    let opt = document.createElement("option")
	    opt.text = name
	    w.append(opt)
	}
	if (typeof initdecoders !== "undefined") initdecoders(w)
    } finally {
	if (h) h.classList.remove("wait")
    }
})

function checkresult() {
    let name = "result="
    let cookies = decodeURIComponent(document.cookie).split(";")
    for (let i = 0; i < cookies.length; i++) {
	let c = cookies[i]
	while (c.charAt(0) == ' ') c = c.substring(1)
	if (c.indexOf(name) == 0) {
	    let code = c.substring(name.length, c.length)
	    let w = document.getElementById("code" + code)
	    if (w) w.style.display = "block"
	    // Delete the cookie so the popup doesn't show again when reloading the page
	    document.cookie = name + ";expires=Thu, 01 Jan 1970 00:00:00 UTC;path=/;SameSite=Strict"
	    setTimeout(hideresults, 10000)
	    return
	}
    }
}

function hideresults() {
    let list = document.getElementsByClassName("result")
    for (let i = 0; i < list.length; i++) {
	hideresult(list[i])
    }
}

function hideresult(w) {
    w.style.display = "none"
}

function famselect(e) {
    if (e.type === "keydown") {
	if (e.key !== "Enter") return
	event.preventDefault()
    } else if (e.type === "input") {
	if (e.inputType !== undefined && e.inputType !== "insertReplacementText") return
    } else {
	console.log(e.type)
	return
    }
    let w = document.getElementById("definition")
    if (w) {
	let value = e.target.value
	if (value in signals) {
	    w.value = signals[value]
	}
	w.focus()
    } 
}

function choose(sel) {
    let value = sel.value
    if (value in signals) {
	setirsignal(signals[value])
    }
}

function getvars(def) {
    // Remove all whitespace
    def = def.replace(/\s/g, "")
    let regexp, matches
    let name, count, skip, min, max

    // Find any use of a variable with bit specifications
    let vars = {}
    regexp = /([A-Z][A-Z0-9]*):(\d+)(?::(\d+))?/g
    while (matches = regexp.exec(def)) {
	[, name, count, skip] = matches
	count = parseInt(count)
	if (skip) count += parseInt(skip)
	max = (1 << count) - 1
	let m = 0
	if (vars[name]) {
	    m = vars[name].max
	} else {
	    vars[name] = {}
	    vars[name].min = 0
	}
	if (max > m) vars[name].max = max
    }

    // Find range specifications and possible default value
    regexp = /([A-Z][A-Z0-9]*):(\d+)\.\.(\d+)(?:=([^,[\]]*))?/g
    while (matches = regexp.exec(def)) {
	[, name, min, max, init] = matches
	if (!vars[name]) vars[name] = {}
	vars[name].min = parseInt(min)
	vars[name].max = parseInt(max)
	vars[name].default = init
    }

    // Find any assignments
    regexp = /([A-Z][A-Z0-9]*)=/g
    while (matches = regexp.exec(def)) {
	name = matches[1]
	if (!vars[name]) vars[name] = {}
	vars[name].assigned = true
    }

    // Special treatment for the toggle bit
    if (vars.T && vars.T.min == 0 && vars.T.max == 1) {
	if (vars.T.default === undefined) {
	    vars.T.default = "Toggle"
	}
    }
    return vars
}

function setirsignal(def) {
    let vars = getvars(def)
    // Hide or show relevant rows
    let t = document.getElementById("list")
    for (let i = 0; i < t.rows.length; i++) {
	let row = t.rows[i]
	let found = row.id.match(/^row([A-Z][A-Z0-9]*)$/)
	if (found) {
	    name = found[1]
	    if (vars[name] && !vars[name].assigned) {
		row.style.display = ""
	    } else {
		row.style.display = "none"
		let w = document.getElementById("var" + name)
		// Stop the field from being submitted
		w.name = ""
	    }
	}
    }
    // Configure the input fields for the variables
    for (name in vars) {
	if (vars[name].assigned) continue
	let row = document.getElementById("row" + name)
	if (!row) {
	    // Create a row for a new variable
	    row = document.createElement("TR")
	    row.id = "row" + name
	    let fld = document.createElement("TD")
	    fld.innerText = "Variable " + name + ":"
	    row.appendChild(fld)
	    fld = document.createElement("TD")
	    let inp = document.createElement("INPUT")
	    inp.type = "number"
	    inp.id = "var" + name
	    inp.size = 5
	    fld.appendChild(inp)
	    row.appendChild(fld)
	    let l = document.getElementById("rowT")
	    let p = l.parentNode
	    p.insertBefore(row, l)
	}
	let inp = document.getElementById("var" + name)
	inp.name = name
	inp.min = vars[name].min
	inp.max = vars[name].max
	let title = "Allowed values: " + inp.min + " .. " + inp.max + "."
	if (vars[name].default) {
	    title += " Default: " + vars[name].default + "."
	    if (inp.value == 0) inp.value = ""
	    inp.required = false
	} else {
	    if (inp.value == "") inp.value = 0
	    inp.required = true
	}
	inp.title = title
    }
}

function transmit(frm) {
    let data = new FormData(frm)
    fetch("transmit.html", {
	method: "POST",
	body: data
    })
    return false
}

function receivemode(w) {
    let data = new FormData()
    data.append("mode", w.value)
    fetch("settings.html", {
	method: "POST",
	body: data
    })
    showcapture()
    return false
}

function showcapture() {
    let w = document.getElementById("mode")
    let d = document.getElementById("capture")
    if (w && d) {
	if (w.value == "capture") {
	    d.style.display = ""
	} else {
	    d.style.display = "none"
	}
    }
}

function receive(evt) {
    let w = document.getElementById("receivelog")
    if (w) {
	let message = JSON.parse(evt.data)
	let str = ""
	let title = ""
	for (let key in message) {
	    let text = message[key]
	    if (key == "analysis") {
		for (let k in message[key]) {
		    if (title != "") title += ", "
		    title += k.charAt(0).toUpperCase() + k.slice(1) + ": " + message[key][k]
		}
		continue
	    }
	    if (key == "times") {
		text = "<a href='#graphic' onclick='waveform(this)'>" + text + "</a>"
	    }
	    if (str != "") str += ", "
	    str += key.charAt(0).toUpperCase() + key.slice(1) + ": " + text
	}
	if (title != "") {
	    str = "<span title='" + title + "'>" + str + "</span>"
	}
	w.innerHTML = str + "<br>" + w.innerHTML
    }
}

function clearlog() {
    let w = document.getElementById("receivelog")
    if (w) {
	w.innerHTML = ""
    }
}

function waveform(e) {
    let list = e.innerHTML.split(',')
    let w = document.getElementById("graphic")
    if (w) {
	let ctx = w.getContext("2d")
	let len = list.length
	// Setting the canvas width clears it
	w.width = list.reduce(function(a, b) {return a + b / 100}, 10)
	ctx.clearRect(0, 0, w.width, w.height);
	ctx.beginPath()
	ctx.moveTo(0, 45.5)
	let x = 5.5
	ctx.lineTo(x, 45.5)
	ctx.lineTo(x, 5.5)
	let level = 1
	for (let i = 0; i < len; i++) {
	    x += list[i] / 100
	    ctx.lineTo(x, level ? 5.5 : 45.5)
	    level = !level
	    ctx.lineTo(x, level ? 5.5 : 45.5)
	}
	x += 5
	ctx.lineTo(x, 45.5)
	ctx.stroke()
    }
    if (analyze) {
	analyze(list)
    }
}

function teardown(evt) {
    if (evt.code != 1001) {
	let w = document.getElementById("socklost")
	if (w) {
	    w.style.display = "block"
	}
    }
}

function connect(name, msgfunc = wsdata) {
    let wsurl = "ws" + document.URL.match("s?://[^?#]+/") + name
    if ("WebSocket" in window) {
	ws = new WebSocket(wsurl)
    } else if ("MozWebSocket" in window) {
	ws = new MozWebSocket(wsurl)
    }
    if (ws) {
	ws.onmessage = msgfunc
	ws.onclose = teardown
    }
}
